@@ -99,6 +99,7 @@ void load_rc_file(int);
 void save_rc_file();
 void MainLoop();
 void GetXPM(char *, char *);
+void MaskSet();
 int  FlushExpose(Window w);
 void newtext (char *);
 void text (char *);
@@ -166,11 +167,6 @@ GC WinGC;
 int CarrierOn = FALSE;
 int screen;
 
-/* 0.12 themes stuff: */
-
-/*#define THDIR "/usr/local/share/AScd"*/
-char theme[60];
-
 /* everything dealing with the hardware volume: */
 
 #define MAX_VOL 255
@@ -201,8 +197,48 @@ int has_clicked = FALSE;
 int info_modified = 0;
 int big_spaces = 0;
 int slow_down = 0;
+
+/* 0.12 themes stuff: */
+
+/*#define THDIR "/usr/local/share/AScd"*/
+char theme[80];
+
+unsigned int theme_select = 0;
+unsigned int theme_select_nbr = 0;
+
+/* 'no mask' is the new redraw mode used in 0.12.1: instead of
+   using a big shape mask based on the background, we combine the
+   background with the buttons shapes. This allows using non rect 
+   buttons. Old mode is no longer supported.
+*/
+unsigned int no_mask = TRUE;
+
+/* the themes level 2 globals. Not yet supported, this is just
+   a draft!
+*/
+
 unsigned int panel = 0;
 
+/*
+#define BMAX 16
+unsigned int panels = 0;
+unsigned int but_nbr = 0;
+
+char th_name[80];
+char th_author[80];
+char th_release[80];
+char th_email[80];
+
+
+XpmIcon but_xpm[BMAX];
+char but_left[BMAX,20];
+char but_mid[BMAX,20];
+char but_right[BMAX,20];
+unsigned int but_x[BMAX];
+unsigned int but_x[BMAX];
+unsigned int but_w[BMAX];
+unsigned int but_h[BMAX];
+*/
 
 /*****************************************************************************/
 
@@ -522,6 +558,8 @@ void save_rc_file()
 void cdplayer_mouse_events(XEvent Event) 
 {
     char txt[127];
+    unsigned int i;
+    FILE *in;
 
     has_clicked = TRUE;
     show_db_pos = 0;
@@ -569,27 +607,74 @@ void cdplayer_mouse_events(XEvent Event)
 
 	    /* the Fast-track select */
 
-	    if ((cur_cdmode == WM_CDM_PLAYING) || (cur_cdmode == WM_CDM_PAUSED)) {
+	    if (theme_select == 0) {
+		if ((cur_cdmode == WM_CDM_PLAYING) || (cur_cdmode == WM_CDM_PAUSED)) {
+		    switch (Event.xbutton.button) {
+		    case 3: /* down */
+			if (fast_track == 0) {
+			    if (cur_track > 1) fast_track = cur_track - 1;
+			    else fast_track = cur_ntracks;
+			} else {
+			    if (fast_track == 1) fast_track = cur_ntracks;
+			    else fast_track--;
+			}
+			break;
+		    case 2:
+			fast_track = 0;
+			break;
+		    default: 
+			if (fast_track == 0) {
+			    if (cur_track < cur_ntracks) fast_track = cur_track + 1;
+			    else fast_track = 1;
+
+			} else {
+			    if (fast_track == cur_ntracks) fast_track = 1;
+			    else fast_track++;
+			}
+			break;
+		    }
+		}
+	    } else { /* theme selection */
 		switch (Event.xbutton.button) {
-		case 3: /* down */
-		    if (fast_track == 0) {
-			if (cur_track > 1) fast_track = cur_track - 1;
-			else fast_track = cur_ntracks;
-		    } else {
-			if (fast_track == 1) fast_track = cur_ntracks;
-			else fast_track--;
+		case 3:
+		    theme_select --;
+		    if (theme_select == 1) theme_select = theme_select_nbr;
+		    sprintf(txt, "%s/.dir", THDIR);
+		    if (in = fopen(txt, "r")) {
+			i = 0;
+			while (fgets(txt, 255,in)) {
+			    i++;
+			    txt[strlen(txt) - 1] = 0;
+			    if (i == theme_select) {
+				break;
+			    }
+			}
+			fclose(in);
+			strcpy(theme, txt);
+			(void)GetXPM(led_color, led_bgcolor);
+			(void)MaskSet();
 		    }
 		    break;
 		case 2:
-		    fast_track = 0;
+		    theme_select = 0;
 		    break;
-		default: 
-		    if (fast_track == 0) {
-			if (cur_track < cur_ntracks) fast_track = cur_track + 1;
-			else fast_track = 1;
-		    } else {
-			if (fast_track == cur_ntracks) fast_track = 1;
-			else fast_track++;
+		default:
+		    theme_select ++;
+		    if (theme_select > theme_select_nbr) theme_select = 1;
+		    sprintf(txt, "%s/.dir", THDIR);
+		    if (in = fopen(txt, "r")) {
+			i = 0;
+			while (fgets(txt, 255,in)) {
+			    i++;
+			    txt[strlen(txt) - 1] = 0;
+			    if (i == theme_select) {
+				break;
+			    }
+			}
+			fclose(in);
+			strcpy(theme, txt);
+			(void)GetXPM(led_color, led_bgcolor);
+			(void)MaskSet();
 		    }
 		    break;
 		}
@@ -599,8 +684,39 @@ void cdplayer_mouse_events(XEvent Event)
 
     else if (Event.xbutton.y < 18) {
 	/* second display line */
-	if (show_db) show_db = FALSE;
-	else show_db = TRUE;
+	if (Event.xbutton.button != 3) {
+	    /* left or middle: toggle track name scrolling on/off */
+	    if (show_db) show_db = FALSE;
+	    else show_db = TRUE;
+	} else {
+	    /* right button: theme selector */
+	    if (theme_select == 0) {
+		theme_select = 1;
+
+		sprintf(txt, "%s/.dir", THDIR);
+		theme_select_nbr = 0;
+		if (in = fopen(txt, "r")) {
+		    while (fgets(txt, 255,in)) theme_select_nbr++;
+		    fclose(in);
+		}
+
+		sprintf(txt, "%s/.dir", THDIR);
+		if (in = fopen(txt, "r")) {
+		    i = 0;
+		    while (fgets(txt, 255, in)) {
+			i++;
+			txt[strlen(txt) - 1] = 0;
+			if (strcmp(txt, theme) == 0) {
+			    theme_select = i;
+			    break;
+			}
+		    }
+		    fclose(in);
+		}
+	    } else {
+		theme_select = 0;
+	    }
+	}
     }
 
     else if ((Event.xbutton.y >= 21) && (Event.xbutton.y <= 25)) {
@@ -648,8 +764,9 @@ void cdplayer_mouse_events(XEvent Event)
 		    panel ++;
 		    if (panel > 1) panel = 0;
 		}
-		RedrawPanel();
-		RedrawWindow(&backXPM);
+		(void)MaskSet();
+		(void)RedrawPanel();
+		(void)RedrawWindow(&backXPM);
 	    }
 	}
     }
@@ -733,6 +850,7 @@ void cdplayer_mouse_events(XEvent Event)
 		newtext("Paused");
 		wanna_play = TRUE;
 		cd_control(PAUSE);
+		(void)MaskSet();
 	    } else {
 		if (cur_cdmode == WM_CDM_PLAYING) {
 		    newtext("Fade Out");
@@ -750,22 +868,26 @@ void cdplayer_mouse_events(XEvent Event)
 		    cd_control(STOPONLY);
 		    wm_cd_status();
 		    cur_track = 0;
+		    (void)MaskSet();
 		} else {
-		    newtext("Ejecting");
+		    newtext("Eject");
 		    cd_control(EJECT);
+		    (void)MaskSet();
 		}
 		redraw = TRUE;
 	    } else {
 		cd_control(INTROSCAN);
 		wm_cd_status();
 		newtext("Intro Sc");
+		(void)MaskSet();
 	    }
 	} else {
 	    if (panel == 0) {
 		if ((cur_cdmode != WM_CDM_EJECTED) && (cur_cdmode != WM_CDM_STOPPED)) {
-		    newtext("Ejecting");
+		    newtext("Eject");
 		    cd_control(EJECT);
 		    fast_track = 0;
+		    (void)MaskSet();
 		} else {
 		    exit(0);
 		}
@@ -803,6 +925,21 @@ void GetXPM(char *foreground, char *background)
     char txt[256];
     XWindowAttributes Attributes;
     int Ret;
+    FILE *in;
+
+    sprintf(txt, "%s/%s", THDIR, theme);
+    if (access(txt, X_OK | R_OK) != 0) {
+	sprintf(txt, "%s/default", THDIR);
+	if (access(txt, X_OK | R_OK) != 0) {
+	    fprintf(stderr, "ascd: fatal error\n\n");
+	    fprintf(stderr, "The '%s' and 'default' themes folders are not \navailable in %s.\n\n", theme, THDIR);
+	    fprintf(stderr, "Please check the permissions and/or redo ascd installation.\n\n");
+	    exit(1);
+	} else {
+	    strcpy(theme, "default");
+	}
+    }
+
 
     XGetWindowAttributes(Disp,Root,&Attributes);
 
@@ -822,32 +959,32 @@ void GetXPM(char *foreground, char *background)
     rewXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/rew.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &rewXPM.pixmap,
-				  &rewXPM.mask, &rewXPM.attributes);
+			      &rewXPM.mask, &rewXPM.attributes);
 
     playXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/play.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &playXPM.pixmap,
-				  &playXPM.mask, &playXPM.attributes);
+			      &playXPM.mask, &playXPM.attributes);
 
     play_onXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/play_on.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &play_onXPM.pixmap,
-				  &play_onXPM.mask, &play_onXPM.attributes);
+			      &play_onXPM.mask, &play_onXPM.attributes);
 
     pauseXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/pause.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &pauseXPM.pixmap,
-				  &pauseXPM.mask, &pauseXPM.attributes);
+			      &pauseXPM.mask, &pauseXPM.attributes);
 
     pause_onXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/pause_on.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &pause_onXPM.pixmap,
-				  &pause_onXPM.mask, &pause_onXPM.attributes);
+			      &pause_onXPM.mask, &pause_onXPM.attributes);
 
     ejectXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/eject.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &ejectXPM.pixmap,
-				  &ejectXPM.mask, &ejectXPM.attributes);
+			      &ejectXPM.mask, &ejectXPM.attributes);
 
     stopXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/stop.xpm", THDIR, theme);
@@ -857,60 +994,65 @@ void GetXPM(char *foreground, char *background)
     lstartXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/lstart.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &lstartXPM.pixmap,
-				  &lstartXPM.mask, &lstartXPM.attributes);
+			      &lstartXPM.mask, &lstartXPM.attributes);
 
     lendXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/lend.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &lendXPM.pixmap,
-				  &lendXPM.mask, &lendXPM.attributes);
+			      &lendXPM.mask, &lendXPM.attributes);
 
     lgoXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/lgo.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &lgoXPM.pixmap,
-				  &lgoXPM.mask, &lgoXPM.attributes);
+			      &lgoXPM.mask, &lgoXPM.attributes);
 
     lgo_onXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/lgo_on.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &lgo_onXPM.pixmap,
-				  &lgo_onXPM.mask, &lgo_onXPM.attributes);
+			      &lgo_onXPM.mask, &lgo_onXPM.attributes);
 
     fadeXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/fade.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &fadeXPM.pixmap,
-				  &fadeXPM.mask, &fadeXPM.attributes);
+			      &fadeXPM.mask, &fadeXPM.attributes);
 
     introXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/intro.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &introXPM.pixmap,
-				  &introXPM.mask, &introXPM.attributes);
+			      &introXPM.mask, &introXPM.attributes);
 
     intro_onXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/intro_on.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &intro_onXPM.pixmap,
-				  &intro_onXPM.mask, &intro_onXPM.attributes);
+			      &intro_onXPM.mask, &intro_onXPM.attributes);
 
     quitXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/quit.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &quitXPM.pixmap,
-				  &quitXPM.mask, &quitXPM.attributes);
+			      &quitXPM.mask, &quitXPM.attributes);
 
     alphaXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/digits.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &alphaXPM.pixmap,
-				  &alphaXPM.mask, &alphaXPM.attributes);
+			      &alphaXPM.mask, &alphaXPM.attributes);
 
     ralphaXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/rdigits.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &ralphaXPM.pixmap,
-				  &ralphaXPM.mask, &ralphaXPM.attributes);
+			      &ralphaXPM.mask, &ralphaXPM.attributes);
 
     barXPM.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
     sprintf(txt, "%s/%s/bar.xpm", THDIR, theme);
     Ret = XpmReadFileToPixmap(Disp, Root, txt, &barXPM.pixmap,
-				  &barXPM.mask, &barXPM.attributes);
+			      &barXPM.mask, &barXPM.attributes);
 
     if (Ret != XpmSuccess) {
-	fprintf(stderr, "ascd: can't load the background and/or buttons pixmaps.\nCauses can be:\n- problem(s) in the files\n- a non existing color in fore/background colors definition\n- not enough free color cells\n\n");
+	fprintf(stderr, "ascd: fatal error\n\n");
+	fprintf(stderr, "Can't load the background and/or buttons pixmaps.\n");
+	fprintf(stderr, "Causes can be:\n");
+	fprintf(stderr, "- problem(s) in the files\n");
+	fprintf(stderr, "- a non existing color in fore/background colors definition\n");
+	fprintf(stderr, "- not enough free color cells\n\n");
 	exit(1);
     }
 } 
@@ -925,7 +1067,6 @@ void CreateWindow(int argc, char **argv)
     unsigned long gcm;
     XTextProperty name;
     Pixel back_pix, fore_pix;
-    /*Pixmap pixmask;*/
     int x_fd;
     int d_depth;
     int ScreenWidth, ScreenHeight;
@@ -1012,6 +1153,9 @@ void CreateWindow(int argc, char **argv)
     
     XMapWindow(Disp, Win);
     
+    if (no_mask) {
+	(void)MaskSet();
+    }
     RedrawWindow(&backXPM);
 } 
 
@@ -1117,31 +1261,32 @@ void show_db_f() {
     char txt[256];
     char txt2[256];
     char dsp[9];
-    if (cd->trk[cur_track - 1].songname != NULL) {
-	if (show_artist) {
-	    if (cd->artist != NULL) {
-		sprintf(txt2, "%s: %s", cd->artist, cd->trk[cur_track - 1].songname);
+    if (theme_select == 0) {
+	if (cd->trk[cur_track - 1].songname != NULL) {
+	    if (show_artist) {
+		if (cd->artist != NULL) {
+		    sprintf(txt2, "%s: %s", cd->artist, cd->trk[cur_track - 1].songname);
+		} else {
+		    strcpy(txt2, cd->trk[cur_track - 1].songname);
+		}
 	    } else {
 		strcpy(txt2, cd->trk[cur_track - 1].songname);
 	    }
+	    Text_Draw("        ", 0, 1, TRUE);
+	    if (strlen(txt2) > 8) {
+		strcpy(txt, "       ");
+		strcat(txt, txt2);
+		if (show_db_pos > strlen(txt)) show_db_pos = 0;
+		tes_sncpy(dsp, txt + show_db_pos, 8);
+		Text_Draw(dsp, 0, 1, TRUE);
+		show_db_pos ++;
+	    } else {
+		Text_Draw(txt2, 0, 1, TRUE);
+	    }
 	} else {
-	    strcpy(txt2, cd->trk[cur_track - 1].songname);
+	    sprintf(txt, "");
 	}
-	Text_Draw("        ", 0, 1, TRUE);
-	if (strlen(txt2) > 8) {
-	    strcpy(txt, "       ");
-	    strcat(txt, txt2);
-	    if (show_db_pos > strlen(txt)) show_db_pos = 0;
-	    tes_sncpy(dsp, txt + show_db_pos, 8);
-	    Text_Draw(dsp, 0, 1, TRUE);
-	    show_db_pos ++;
-	} else {
-	    Text_Draw(txt2, 0, 1, TRUE);
-	}
-    } else {
-	sprintf(txt, "");
     }
-
 }
 
 void RedrawPanel() {
@@ -1252,6 +1397,80 @@ void RedrawPanel() {
     }
 }
 
+void MaskSet() {
+
+    XShapeCombineMask(Disp, Win, ShapeBounding, 0, 0, backXPM.mask, ShapeSet);
+    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 0, 0, backXPM.mask, ShapeSet);
+
+    if (panel == 0) {
+
+	XShapeCombineMask(Disp, Win, ShapeBounding,      0, 27, rewXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding,  0, 27, rewXPM.mask, ShapeUnion);
+
+	XShapeCombineMask(Disp, Win, ShapeBounding,     38, 27, fwdXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding, 38, 27, fwdXPM.mask, ShapeUnion);
+	
+	if (cur_cdmode == WM_CDM_PLAYING) {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 27, play_onXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 27, play_onXPM.mask, ShapeUnion);
+	} else {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 27, playXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 27, playXPM.mask, ShapeUnion);
+	}
+	
+	if (cur_cdmode == WM_CDM_PAUSED) {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,      0, 42, pause_onXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding,  0, 42, pause_onXPM.mask, ShapeUnion);
+	} else {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,      0, 42, pauseXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding,  0, 42, pauseXPM.mask, ShapeUnion);
+	}
+
+	if ((cur_cdmode != WM_CDM_STOPPED) && (cur_cdmode != WM_CDM_EJECTED)) {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 42, stopXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 42, stopXPM.mask, ShapeUnion);
+
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     38, 42, ejectXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 38, 42, ejectXPM.mask, ShapeUnion);
+	} else {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 42, ejectXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 42, ejectXPM.mask, ShapeUnion);
+
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     38, 42, quitXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 38, 42, quitXPM.mask, ShapeUnion);
+	}
+    } else {
+	XShapeCombineMask(Disp, Win, ShapeBounding,      0, 27, lstartXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding,  0, 27, lstartXPM.mask, ShapeUnion);
+
+	XShapeCombineMask(Disp, Win, ShapeBounding,     38, 27, lendXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding, 38, 27, lendXPM.mask, ShapeUnion);
+
+	if (loop_mode) {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 27, lgo_onXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 27, lgo_onXPM.mask, ShapeUnion);
+	} else {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 27, lgoXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 27, lgoXPM.mask, ShapeUnion);
+	}
+
+	
+	XShapeCombineMask(Disp, Win, ShapeBounding,      0, 42, fadeXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding,  0, 42, fadeXPM.mask, ShapeUnion);
+
+	if (intro_mode) {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 42, intro_onXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 42, intro_onXPM.mask, ShapeUnion);
+	} else {
+	    XShapeCombineMask(Disp, Win, ShapeBounding,     19, 42, introXPM.mask, ShapeUnion);
+	    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 19, 42, introXPM.mask, ShapeUnion);
+	}
+
+	XShapeCombineMask(Disp, Win, ShapeBounding,     38, 42, quitXPM.mask, ShapeUnion);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding, 38, 42, quitXPM.mask, ShapeUnion);
+    }
+}
+
 /* ------------------------------------------------------------------------
    The big deal: We have to redraw the pixmaps, and update all leds. But
    there a few cd_control() calls also.
@@ -1263,7 +1482,9 @@ void RedrawWindow(XpmIcon *Icon) {
     static flash=0;
     long int dodo = 300000;
     char cdtime[6];
+    char txt[40];
     int i;
+    FILE *in;
 
     if (strlen(led_text) > 0) {
 	if (text_start == 0) {
@@ -1420,10 +1641,12 @@ void RedrawWindow(XpmIcon *Icon) {
     FlushExpose(Win);
     FlushExpose(Iconwin);
 
-    XShapeCombineMask(Disp, Win, ShapeBounding, 0, 0,
-		      backXPM.mask, ShapeSet);
-    XShapeCombineMask(Disp, Iconwin, ShapeBounding, 0, 0,
-		      backXPM.mask, ShapeSet);
+    if (! no_mask) {
+	XShapeCombineMask(Disp, Win, ShapeBounding, 0, 0,
+			  backXPM.mask, ShapeSet);
+	XShapeCombineMask(Disp, Iconwin, ShapeBounding, 0, 0,
+			  backXPM.mask, ShapeSet);
+    }
 
     XCopyArea(Disp,backXPM.pixmap,Win,WinGC,
 	      0,0,backXPM.attributes.width, backXPM.attributes.height,0,0);
@@ -1454,6 +1677,38 @@ void RedrawWindow(XpmIcon *Icon) {
 	return;
     }
 
+    /* Theme select
+       ============
+    */
+
+    if (theme_select > 0) {
+	Text_Draw("THEME", 0, 0, TRUE);
+	sprintf(cdtime, "%02d", theme_select);
+	Text_Draw(cdtime, 6, 0, TRUE);
+	strcpy(cdtime, " ");
+	cdtime[0] = 127;
+	Text_Draw(cdtime, 5, 0, FALSE);
+	
+	i = 0;
+	sprintf(txt, "%s/.dir", THDIR);
+	if (in = fopen(txt, "r")) {
+	    while (fgets(txt, 255,in)) {
+		i++;
+		txt[strlen(txt) - 1] = 0;
+		if (i == theme_select) {
+		    Text_Draw(txt, 0, 1, FALSE);
+		    break;
+		}
+	    }
+	    fclose(in);
+	}
+	return;
+    }
+
+    /* Fast Track select
+       =================
+    */
+
     if (fast_track == 0) {
 	if (cur_track > 0) {
 	    if (time_mode != 3) {
@@ -1578,6 +1833,7 @@ void MainLoop()
 		    cd_control(PLAY);
 		    wm_cd_status();
 		    dodo = RDTIME;
+		    (void)MaskSet();
 		    RedrawWindow(&backXPM);
 		}
 	    }
@@ -1715,15 +1971,12 @@ int main(int argc, char *argv[])
     if (strlen(led_color) == 4) strcat(led_color, DEFAULT_COLOR);
     if (strlen(led_bgcolor) == 4) strcat(led_bgcolor, DEFAULT_BGCOLOR);
 
-
 #ifdef WMK
     WMInitializeApplication("AScd", &argc, argv);
 #endif
     CreateWindow(argc, argv);
 
     wm_cd_status();
-
-
     
     if (cur_cdmode != WM_CDM_EJECTED) {
 	volume = read_initial_volume(max_volume);
@@ -1739,6 +1992,8 @@ int main(int argc, char *argv[])
 	wm_cd_status();
 	RedrawWindow(&backXPM);
     }
-    MainLoop();
+
+    (void)MaskSet();
+    (void)MainLoop();
     return 0;
-} 
+}
